package com.perforce.bumble.diff;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import com.perforce.bumble.connection.DefaultService;
import com.perforce.p4java.core.file.FileSpecBuilder;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.option.server.GetFileContentsOptions;
import com.perforce.p4java.server.IOptionsServer;

import difflib.DiffUtils;
import difflib.Patch;

public class DiffService extends DefaultService {

	public DiffService(IOptionsServer p4) throws P4JavaException {
		super(p4);
	}

	public List<String> getDiff(String path) throws IOException, P4JavaException {
		String root = iclient.getRoot() + "/";
		File local = new File(root + path);

		List<String> yours = fileToLines(local);
		List<String> base = p4ToLines(root + path);

		Patch<String> patch = DiffUtils.diff(base, yours);
		List<String> diff = DiffUtils.generateUnifiedDiff("base", path, base, patch, 3);
		return diff;
	}

	private List<String> fileToLines(File file) throws IOException {
		BufferedReader in = new BufferedReader(new FileReader(file));
		List<String> lines = new ArrayList<String>();

		String line;
		while ((line = in.readLine()) != null) {
			lines.add(line);
		}

		in.close();
		return lines;
	}

	private List<String> p4ToLines(String path) throws IOException, P4JavaException {
		String rev = path + "#have";
		List<IFileSpec> file = FileSpecBuilder.makeFileSpecList(rev);
		GetFileContentsOptions printOpts = new GetFileContentsOptions();
		printOpts.setNoHeaderLine(true);

		InputStream ins = p4.getFileContents(file, printOpts);
		BufferedReader in = new BufferedReader(new InputStreamReader(ins));
		List<String> lines = new ArrayList<String>();

		String line;
		while ((line = in.readLine()) != null) {
			lines.add(line);
		}

		in.close();
		return lines;
	}
}
