package com.perforce.bumble.project;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map.Entry;
import java.util.Properties;

public class Project {

	private String name;
	private String path;
	
	transient private Properties config;

	public Project(String name, String path) throws IOException {
		this.name = name;
		this.path = path;
		this.config = load();
	}

	private Properties load() throws IOException {
		Properties config = new Properties();
		String p4config = path + "/.p4config";

		FileInputStream in = new FileInputStream(p4config);
		config.load(in);
		in.close();

		return config;
	}

	public void save() throws IOException {
		String p4config = path + "/.p4config";
		FileWriter writer = new FileWriter(p4config);
		BufferedWriter out = new BufferedWriter(writer);

		StringBuffer sb = new StringBuffer();
		for (Entry<Object, Object> entry : config.entrySet()) {
			String key = (String) entry.getKey();
			String value = (String) entry.getValue();
			sb.append(key + "=" + value + "\n");
		}

		out.write(sb.toString());
		out.close();
	}

	public String getName() {
		return name;
	}

	public File getPath() {
		return new File(path);
	}

	public void setPath(File path) {
		this.path = path.getAbsolutePath();
	}

	public String getConfig(String key) {
		return config.getProperty(key);
	}

	public void setConfig(String key, String value) {
		config.setProperty(key, value);
	}

	public String getURI() {
		if (config.containsKey("P4INITROOT")) {
			return dvcsToURI();
		}
		return getP4JavaUri();
	}

	private String dvcsToURI() {
		String rsh = "/usr/local/bin/p4d ";
		rsh += "-r " + path + "/.p4root ";
		rsh = "p4jrsh://" + rsh + "-i --java";
		return rsh;
	}

	private String getP4JavaUri() {
		String port = config.getProperty("P4PORT");
		if (port.startsWith("ssl:")) {
			String trim = port.substring(4, port.length());
			return "p4javassl://" + trim;
		}
		if (port.startsWith("rsh:")) {
			String trim = port.substring(4, port.length());
			return "p4jrsh://" + trim + " --java";
		}
		if (port.contains(":")) {
			return "p4java://" + port;
		}
		return "p4java://localhost:" + port;
	}
}
